'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// TODO:
// - Update FieldState BurstAspectRatio on resize
// - Aria
// - IE Tests

// Global
var FIELD_STATE = {
  ACTIVE: 'ACTIVE',
  FAIL: 'ERROR',
  PASS: 'PASS'
};

var SvgDefs = {
  filters: {
    blur: {
      active: document.querySelector('#blur-active feGaussianBlur'),
      error: document.querySelector('#blur-error feGaussianBlur')
    }
  }
};

// Components

var Form = function () {
  function Form(form) {
    var _this = this;

    _classCallCheck(this, Form);

    this.activeField = 0;
    this.expanded = false;
    this.submitted = false;
    this.submit = this.submit.bind(this);
    this.elements = {
      form: form,
      fields: [].slice.call(form.querySelectorAll('[data-js~=form-field]')),
      button: form.querySelector('[data-js~=form-button]'),
      pagination: form.querySelector('[data-js~=form-pagination]'),
      response: form.querySelector('[data-js~=form-response]'),
      responseMask: form.querySelector('[data-response-mask]'),
      expand: form.querySelector('[data-js~=form-expand]')
    };

    this.fields = this.elements.fields.map(function (f, i) {
      i === 0 ? f.classList.add('is-active') : f.classList.add('is-next');
      return new Field(_this, f, i);
    });

    this.pagination = new Pagination(this, this.elements.pagination);

    this.arrow = new Arrow(this, this.elements.button);

    this.bursts = new Array(8).fill(0).map(function (x, i) {
      return createBurst(_this.elements.response, i);
    });

    this.validator = new Validator(this);

    this.successTl = new mojs.Timeline().add(new mojs.Tween({
      duration: 600,
      easing: 'sin.in',
      onUpdate: updateFadeMask.bind(null, this.elements.responseMask),
      onComplete: function onComplete() {
        _this.elements.response.classList.add('is-active');
        _this.bursts.forEach(function (x) {
          return x.play();
        });
      }
    }));

    form.addEventListener('submit', this.submit);

    this.elements.expand.addEventListener('click', this.expand.bind(this));
  }

  Form.prototype.next = function next() {
    this.enableField(this.activeField + 1);
  };

  Form.prototype.enableField = function enableField(i) {
    if (i == this.activeField || i < 0 || i >= this.fields.length) return false;
    this.pagination.activateDot(i);
    this.activeField = parseInt(i);
    this.arrow.change();
    this.fields.forEach(function (f, j) {
      return i == j ? f.enable() : f.disable();
    });
  };

  Form.prototype.submit = function submit(e) {
    e.preventDefault();
    if (!this.expanded && this.activeField < this.fields.length - 1) {
      if (this.fields[this.activeField].isValid()) {
        this.next();
      }
    } else if (this.fields.every(function (f) {
      return f.isValid();
    })) {
      this.success();
    }
  };

  Form.prototype.success = function success() {
    this.arrow.submit();
    this.pagination.disable();
    this.elements.form.classList.remove('is-expanded');
    this.fields[this.activeField].disable();
    this.activeField = null;
    this.successTl.play();
  };

  Form.prototype.expand = function expand(e) {
    this.expanded = true;
    e.preventDefault();
    this.elements.form.classList.add('is-expanded');
    this.fields.forEach(function (f) {
      return f.enable();
    });
    this.pagination.disable();
  };

  return Form;
}();

var Pagination = function () {
  function Pagination(form, pagination) {
    var _this2 = this;

    _classCallCheck(this, Pagination);

    this.jump = this.jump.bind(this);

    this.form = form;
    this.elements = {
      pagination: pagination,
      dots: [].slice.call(pagination.querySelectorAll('[data-js~=form-dot]'))
    };

    this.elements.dots.forEach(function (el, i) {
      el.setAttribute('data-index', i);
      el.addEventListener('click', _this2.jump);
    });
  }

  Pagination.prototype.jump = function jump(e) {
    var i = e.currentTarget.getAttribute('data-index');
    this.form.enableField(i);
  };

  Pagination.prototype.activateDot = function activateDot(i) {
    this.elements.dots[this.form.activeField].classList.remove('is-active');
    if (!isNaN(i) && i >= 0) {
      this.elements.dots[i].classList.add('is-active');
    }
  };

  Pagination.prototype.disable = function disable() {
    this.elements.pagination.classList.add('is-disabled');
  };

  return Pagination;
}();

var Field = function () {
  function Field(form, field, i) {
    var _this3 = this;

    _classCallCheck(this, Field);

    this.form = form;
    this.id = i;
    this.dirty = false;
    this.elements = {
      field: field,
      ctrl: field.querySelector('input'),
      svg: field.querySelector('svg'),
      note: field.querySelector('[data-js~=field-note]')
    };

    var _elements$svg$getBoun = this.elements.svg.getBoundingClientRect();

    var width = _elements$svg$getBoun.width;
    var height = _elements$svg$getBoun.height;

    this.burstRatio = {
      x: width / 100,
      y: height / 100
    };

    this.states = {
      active: new FieldState(form, this, FIELD_STATE.ACTIVE),
      fail: new FieldState(form, this, FIELD_STATE.FAIL)
    };

    this.elements.ctrl.addEventListener('focus', function (e) {
      _this3.form.enableField(_this3.id);
      _this3.states.active.enable();
    });

    this.elements.ctrl.addEventListener('blur', this.states.active.disable);

    this.elements.ctrl.addEventListener('keyup', function (e) {
      if (_this3.dirty) {
        _this3.isValid();
      } else if (_this3.elements.ctrl.value.length > 0) {
        _this3.dirty = true;
      }
    });
  }

  Field.prototype.enable = function enable() {
    this.elements.field.classList.add('is-active');
    this.elements.ctrl.focus();
  };

  Field.prototype.disable = function disable() {
    this.elements.field.classList.remove('is-active', 'is-previous', 'is-next');
    this.elements.field.classList.add(this.id < this.form.activeField ? 'is-previous' : 'is-next');
  };

  Field.prototype.isValid = function isValid() {
    var results = this.form.validator.validateField(this.elements.ctrl);
    if (typeof results !== "boolean") {
      // Get or create error
      console.log(results);
      this.elements.note.children[1].textContent = results[0];
      this.elements.note.classList.add('is-visible');
      this.states.fail.enable();
    } else {
      this.elements.note.classList.remove('is-visible');
      this.states.fail.disable();
    }
    return !results;
  };

  return Field;
}();

var FieldState = function () {
  function FieldState(form, field, state) {
    var _this4 = this;

    _classCallCheck(this, FieldState);

    this.enable = this.enable.bind(this);
    this.disable = this.disable.bind(this);

    this.form = form;
    this.enabled = false;
    this.field = field;
    this.svg = field.elements.svg;
    this.state = state.toLowerCase();
    this.mask = this.svg.querySelector('[data-state-mask=' + this.state + ']');
    this.burst = this.svg.querySelector('[data-state-burst=' + this.state + ']');

    this.tl = new mojs.Timeline();

    // Fade In
    this.tl.add(new mojs.Tween({
      duration: 350,
      easing: 'sin.in',
      onUpdate: updateFadeMask.bind(null, this.mask)
    }));

    // Burst out
    this.tl.add(new mojs.Tween({
      delay: 350,
      duration: 350,
      onStart: function onStart(isForward) {
        return isForward || (_this4.burst.style.opacity = 0);
      },
      onUpdate: updateBurst.bind(null, this.burst, this.field.burstRatio, SvgDefs.filters.blur[this.state])
    }));

    this.burst.style.opacity = 0;
  }

  FieldState.prototype.enable = function enable() {
    if (!this.enabled) {
      this.enabled = true;
      this.tl.play();
    }
  };

  FieldState.prototype.disable = function disable() {
    if (this.enabled) {
      this.enabled = false;
      this.tl.playBackward();
    }
  };

  return FieldState;
}();

var Arrow = function () {
  function Arrow(form, button) {
    _classCallCheck(this, Arrow);

    this.form = form;
    this.elements = {
      button: button,
      arrow: button.querySelector('g')
    };

    this._change = new mojs.Html({
      el: this.elements.arrow,
      duration: 700,
      easing: mojs.easing.path('M0, 100 C0, 100 30, 0 30, 0 C30, 0 50, 0 50, 0 C50, 0 50, 205 50, 205 C50, 205 70, 205 70, 205 C70, 205 100, 100 100, 100'),
      x: { 0: '100%' }
    });

    this._submit = new mojs.Html({
      el: this.elements.arrow,
      duration: 350,
      easing: mojs.easing.path('M0, 100 C0, 100 12.86, 121.07 25, 115 C37.14, 108.93 58.18, 2.96 100, 0'), //arrowSubmitCurve.getEasing(),
      x: { 0: '100%' }
    });
  }

  Arrow.prototype.change = function change() {
    this._change.play();
  };

  Arrow.prototype.submit = function submit() {
    this._submit.play();
  };

  return Arrow;
}();

var FileUpload = function () {
  function FileUpload(field, options) {
    _classCallCheck(this, FileUpload);

    this.options = Object.assign({
      allowedFileTypes: []
    }, options);

    this.elements = {
      field: field,
      ctrl: field.querySelector('input'),
      label: field.querySelector('span')
    };

    this.multiple = this.elements.ctrl.multiple;
    this.placeholderTxt = this.elements.label.textContent;

    document.addEventListener('dragover', this.dragover);
    document.addEventListener("dragenter", this.dragenter.bind(this));
    document.addEventListener("dragleave", this.dragleave.bind(this));
    document.addEventListener('drop', this.drop.bind(this));

    this.elements.ctrl.addEventListener('change', this.change.bind(this));
  }

  FileUpload.prototype.dragover = function dragover(e) {
    if (e.target.className == "FileUpload-label") {
      e.preventDefault();
    }
  };

  FileUpload.prototype.dragenter = function dragenter(e) {
    // highlight potential drop target when the draggable element enters it
    if (e.target.className == "FileUpload-label") {
      this.elements.field.classList.add('is-highlighted');
      this.elements.ctrl.focus();
    }
  };

  FileUpload.prototype.dragleave = function dragleave(e) {
    // reset background of potential drop target when the draggable element leaves it
    if (e.target.className == "FileUpload-label") {
      this.elements.field.classList.remove('is-highlighted');
      this.elements.ctrl.blur();
    }
  };

  FileUpload.prototype.drop = function drop(e) {
    var _this5 = this;

    e.preventDefault();
    this.elements.field.classList.remove('is-highlighted');
    if (e.dataTransfer.types.includes("Files")) {
      if (e.dataTransfer.files.length > 1 && !this.multiple) {
        alert("Only one file may be uploaded");
        return;
      }
      if ([].slice.call(e.dataTransfer.files).some(function (x) {
        return _this5.options.allowedFileTypes.length && !_this5.options.allowedFileTypes.includes(x);
      })) {
        alert("File type not supported!");
        return;
      }
      this.elements.ctrl.files = e.dataTransfer.files;
    }
  };

  FileUpload.prototype.change = function change(e) {
    var count = this.elements.ctrl.files.length;
    if (count) {
      var files = [].slice.call(this.elements.ctrl.files).reduce(function (accum, curr) {
        return accum += curr.name, accum;
      }, "");
      this.elements.label.classList.add('has-value');
      this.elements.label.textContent = count + ' file' + (count == 1 ? "s" : "") + ' selected (' + files + ')';
    } else {
      this.elements.label.classList.remove('has-value');
      this.elements.label.textContent = this.placeholderTxt;
    }
  };

  return FileUpload;
}();

var Validator = function () {
  function Validator(form) {
    _classCallCheck(this, Validator);

    this.form = form;
    this.fields = form.fields;
    this.constraints = this.fields.reduce(this.buildConstraint, {});
  }

  Validator.prototype.buildConstraint = function buildConstraint(constraints, item) {
    var field = item.elements.ctrl;
    var name = field.name || field.getAttribute('name');

    constraints[name] = {};

    // Don't try to validate disabled fields
    if (!field.hasAttribute('disabled')) {
      // Required Validation
      if (field.hasAttribute('data-val-required')) {
        var msg = field.getAttribute('data-val-required');
        constraints[name].presence = msg ? {
          message: "^" + msg
        } : true;
      }

      // Regex Validation
      if (field.hasAttribute('data-val-regex')) {
        constraints[name].format = {
          pattern: field.getAttribute('data-regex'),
          message: "^" + field.getAttribute('data-val-regex')
        };
      }
    }

    return constraints;
  };

  Validator.prototype.validateField = function validateField(field) {
    var name = field.name || field.getAttribute('name');
    console.log(field.value, this.constraints[name]);
    return this.constraints[name] ? validate.single(field.value.trim() || null, this.constraints[name]) || false : false;
  };

  Validator.prototype.validate = function (_validate) {
    function validate() {
      return _validate.apply(this, arguments);
    }

    validate.toString = function () {
      return _validate.toString();
    };

    return validate;
  }(function () {
    return validate(this.form.elements.form, this.constraints);
  });

  return Validator;
}();

// Utils

var updateFadeMask = function updateFadeMask(el, e) {
  var delta = e * 200;
  el.setAttribute('x1', delta - 100);
  el.setAttribute('x2', delta);
};

var updateBurst = function updateBurst(el, ratio, blur, e, p, isForward) {
  if (isForward) {
    el.style.opacity = (1 - e) / 2;
    el.setAttribute('x', e * (-5 * ratio.y));
    el.setAttribute('y', e * (-5 * ratio.x));
    el.setAttribute('width', 100 + e * 10 * ratio.y);
    el.setAttribute('height', 100 + e * 10 * ratio.x);
    blur.setAttribute('stdDeviation', e * 5);
  }
};

function createBurst(parent, i) {
  var delay = Math.round(Math.random() * 500);
  return new mojs.Burst({
    parent: parent,
    top: 'rand(0%, 100%)',
    left: 'rand(0%, 100%)',
    radius: { 0: 100 },
    count: 15,
    degree: 45,
    angle: 45 * i,
    delay: 'rand(0, 500)',
    onComplete: function onComplete() {
      this.generate();
    },

    children: {
      // property map - maps over children with mod function
      shape: ['circle'],
      // points: 'rand(3, 5)',
      radius: 2,
      // property map - maps over children with mod function
      fill: ['#999', '#ff4646', '#ff1717'],
      angle: { 0: 180 },
      // rand string - generates random value for every child rand( min, max )
      // stagger string( start, step ) for every child
      delay: 'rand(' + delay + ', ' + (delay + 300) + ')',
      swirlSize: 'rand(0, 10)',
      isSwirl: true,
      pathScale: 'rand(0.8, 2)',
      swirlFrequency: 'rand(0, 7)',
      duration: 750
    },
    // opacity: { 0 : 1 },
    timeline: {
      delay: delay,
      repeat: 5
    }
  });
}

function nonScalingStroke(els) {
  if (els.some(function (el) {
    return !('vector-effect' in el.style);
  })) {
    els.forEach(function (el) {
      var scaled = [].slice.call(el.querySelectorAll('[vector-effect="non-scaling-stroke"]'));
      var width = el.clientWidth;
      var height = el.clientHeight;

      scaled.forEach(function (x) {
        var stroke = parseFloat(x.getAttribute('stroke-width'));
        var clone = x.parentNode.appendChild(x.cloneNode());
        x.setAttribute('stroke-dasharray', '100 100 100 100');
        x.setAttribute('stroke-width', stroke / (height / 100));
        clone.setAttribute('stroke-dasharray', '0 100 100 100 100');
        clone.setAttribute('stroke-width', stroke / (width / 100));
        clone.setAttribute('y', parseInt(clone.getAttribute('y')) + 1);
        clone.setAttribute('height', parseInt(clone.getAttribute('height')) - 2);
      });
    });
  }
}

// GO!
var scalingStrokePolyfill = nonScalingStroke([].slice.call(document.querySelectorAll('svg')));

var form = new Form(document.querySelector('[data-js~=form]'));
var cv = new FileUpload(document.querySelector('[data-js~=upload]'));